library ieee;
use ieee.std_logic_1164.all;


entity opb_i2c_controller is  -- USER --
  generic
  (
    C_OPB_AWIDTH       : integer          := 32;
    C_OPB_DWIDTH       : integer          := 32;
    C_BASEADDR         : std_logic_vector := X"FEFF0200";
    C_HIGHADDR         : std_logic_vector := X"FEFF02FF");
  
  port
  (
    --Required OPB bus ports, do not add to or delete
    OPB_ABus     : in  std_logic_vector(0 to C_OPB_AWIDTH-1);
    OPB_BE       : in  std_logic_vector(0 to C_OPB_DWIDTH/8-1);
    OPB_Clk      : in  std_logic;
    OPB_DBus     : in  std_logic_vector(0 to C_OPB_DWIDTH-1);
    OPB_RNW      : in  std_logic;
    OPB_Rst      : in  std_logic;
    OPB_select   : in  std_logic;
    OPB_seqAddr  : in  std_logic;
    I2C_DBus     : out std_logic_vector(0 to C_OPB_DWIDTH-1);
    I2C_errAck   : out std_logic;
    I2C_retry    : out std_logic;
    I2C_toutSup  : out std_logic;
    I2C_xferAck  : out std_logic;

    -- USER --
    SCL      : inout std_logic;
    SDA      : inout std_logic    
  );
 
end entity opb_i2c_controller; --USER-- 

-------------------------------------------------------------------------------
-- architecture
-------------------------------------------------------------------------------

architecture imp of opb_i2c_controller is --USER-- 

component IOBUF_F_12
 port (
    O : out STD_ULOGIC;
    IO : inout STD_ULOGIC;
    I : in STD_ULOGIC;
    T : in STD_ULOGIC);
end component;
 
signal wdata : std_logic_vector(0 to 7);
signal rdata : std_logic_vector(0 to 7);
signal rnw  : std_logic;
signal cs, xfer : std_logic;
signal q0,q1 : std_logic;
signal i2c_din : std_logic;

begin


sda_pad : IOBUF_F_12 port map (
  I => wdata(0),
  IO => SDA,
  O =>  i2c_din,
  T => wdata(1)
);

scl_pad : IOBUF_F_12 port map (
  I => wdata(2),
  IO => SCL,
  O => open,
  T => wdata(3)
);
  
-- Chip select, memory mapped. XIoOut8 for selecting the I2C controller
process (OPB_select, OPB_ABus)
begin

  if(OPB_select='1' and OPB_ABus(0 to 23)=C_BASEADDR(0 to 23)) then
    cs <= '1'; else
    cs <= '0';
  end if;  
end process;
  

-- I2C Bus SDA interconnection
process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst='1') then
    wdata <= "11111111";
    rdata <= "00000000";
    
  elsif OPB_Clk'event and OPB_Clk = '1' then
    rnw <= OPB_RNW;

    if (q1 = '0' and q0 = '1' and rnw='0') then
       wdata <= OPB_DBus(0 to 7);
    end if;

    if (q1='1') then
       rdata <= "00000000";
    elsif (q1='0' and q0='1' and rnw = '1') then
       rdata <= i2c_din & "0000000";
    end if;    

  end if;
end process;


process (OPB_Clk,OPB_Rst)
begin
  if (OPB_Rst = '1') then
    q0 <= '0';
    q1 <= '0';
  elsif OPB_Clk'event and OPB_Clk='1' then
    q1 <= not q1 and q0;
    q0 <= not q1 and not q0 and cs;
  end if;
end process;

xfer <= q1;
I2C_xferAck <= xfer;
I2C_DBus <= rdata & X"000000";
I2C_errAck <= '0';
I2C_retry <= '0';
I2C_toutSup <= '0';

end architecture imp;
